// convenience_ext.h
//


// This file implements two additional functions:
//
// 1) Compare a given ::boost::filesystem::path object with a file pattern
//
// bool ::boost::filesystem::is_match(const fs::path & ph, const char *pattern)
//
//
// 2) Get the file dates for a given file represented by a path object
//
// filedates		file_dates(const fs::path & ph);


#ifndef BOOST_FILESYSTEM_CONVENIENCE_EXT_HPP
#define BOOST_FILESYSTEM_CONVENIENCE_EXT_HPP

#ifdef _MSC_VER
#	pragma warning(disable:4800)
#endif

#include <iostream>

#include <boost/filesystem/operations.hpp>
#include <boost/filesystem/fstream.hpp>
#include <boost/filesystem/exception.hpp>


#ifdef BOOST_FILESYSTEM_FILEDATES

	#include <boost/date_time/posix_time/posix_time.hpp>
	#include <boost/date_time/gregorian/gregorian.hpp>

	// VC++ has non standard open(), close() and fstat() methods if not compiled
	// with /Za (which of course will not work if you use the Dinkumware STL
	//
	// Havn't tried yet, but it is possible that /Za works with VC++ 7.0 and later
	//
	#if defined(BOOST_MSVC) && !defined(__STDC__)

		#include <io.h>
		#include <fcntl.h>
		#include <sys/stat.h>
		#include <sys/types.h>

		inline int open(const char * szfn, int	iflags, ...)	{ return _open(szfn, iflags); }
		inline int close(int	ifd)														{ return _close(ifd); }
		inline int fstat(int ifd, struct stat * ps)						{ return _fstat(ifd, (struct _stat *) ps); }

	#else

		#include <fcntl.h>
		#include <sys/stat.h>
		#include <sys/types.h>

	#endif

#endif // BOOST_FILESYSTEM_FILEDATES


// In WIN32 we have case insensitive file names, so the pattern-matcher must
// compare file names case insensitive
//
#ifdef WIN32
	#define BOOST_FILENAME_CHAR(x) toupper(x)
#else
	#define BOOST_FILENAME_CHAR(x) x
#endif

namespace boost
{
	namespace filesystem
	{

#ifdef BOOST_FILESYSTEM_FILEDATES

		//! The boost::filesystem::filedates structure is used to store the file dates, created, modified and last accessed
		/*! This structure is filled and returned by the boost::filesystem::file_dates() function.
				To use filedates, define the BOOST_FILESYSTEM_FILEDATES macro and link your program
				with the boost::date_time library.
		*/
		struct filedates
		{
			boost::posix_time::ptime	m_file_created;
			boost::posix_time::ptime	m_file_last_accessed;
			boost::posix_time::ptime	m_file_last_modified;

			filedates()
				: m_file_created(boost::date_time::neg_infin),
					m_file_last_accessed(boost::date_time::neg_infin),
					m_file_last_modified(boost::date_time::neg_infin)
			{
			}
		};

		//! file_dates() returns a ::boost::filesystem::filedates structure containing the dates
		/*! The pattern allows for the wildchars '*' and '?' which behave as expected.
				Comparison is only performed on path.leaf() and the pattern.

				@param	ph				The ::boost::filesystem::path object

				@param	pattern		Const character string containing the file pattern

				@return bool			If true the path object passed in matches the pattern else it doesn't

				\note The function will throw if path.is_directory() == true
		*/
		filedates		file_dates(const path & ph);

#endif // BOOST_FILESYSTEM_FILEDATES


		//! is_match() compares a boost::filesystem::path object against a pattern.
		/*! The pattern allows for the wildchars '*' and '?' which behave as expected.
				Comparison is only performed on path.leaf() and the pattern.

				@param	ph				The ::boost::filesystem::path object

				@param	pattern		Const character string containing the file pattern

				@return bool			If true the path object passed in matches the pattern else it doesn't

				\note The function will throw if path.is_directory() == true
		*/
		bool				is_match(const path & ph, const char *pattern);
	}
}


#endif /* BOOST_FILESYSTEM_CONVENIENCE_EXT_HPP */

